@@ -18,4 +18,4 @@ y <- x + 0.25*x2 + id.eff[id] + firm.eff[firm] + rnorm(length(x))
 ## estimate
 summary(est <- felm(y ~ x+x2 + G(id) + G(firm)))
 getfe(est)
-summary(lm(y ~ x + x2 + id + firm -1))
+summary(lm(y ~ x + x2 + id + firm))
